package models

// Code generated by xo. DO NOT EDIT.

import (
	"context"
)

// Mapping represents a row from 'mapping'.
type Mapping struct {
	TargetID string `json:"target_id"` // target_id
	SourceID string `json:"source_id"` // source_id
	// xo fields
	_exists, _deleted bool
}

// Exists returns true when the Mapping exists in the database.
func (m *Mapping) Exists() bool {
	return m._exists
}

// Deleted returns true when the Mapping has been marked for deletion from
// the database.
func (m *Mapping) Deleted() bool {
	return m._deleted
}

// Insert inserts the Mapping to the database.
func (m *Mapping) Insert(ctx context.Context, db DB) error {
	switch {
	case m._exists: // already exists
		return logerror(&ErrInsertFailed{ErrAlreadyExists})
	case m._deleted: // deleted
		return logerror(&ErrInsertFailed{ErrMarkedForDeletion})
	}
	// insert (manual)
	const sqlstr = `INSERT INTO mapping (` +
		`target_id, source_id` +
		`) VALUES (` +
		`$1, $2` +
		`)`
	// run
	logf(sqlstr, m.TargetID, m.SourceID)
	if _, err := db.ExecContext(ctx, sqlstr, m.TargetID, m.SourceID); err != nil {
		return logerror(err)
	}
	// set exists
	m._exists = true
	return nil
}

// Update updates a Mapping in the database.
func (m *Mapping) Update(ctx context.Context, db DB) error {
	switch {
	case !m._exists: // doesn't exist
		return logerror(&ErrUpdateFailed{ErrDoesNotExist})
	case m._deleted: // deleted
		return logerror(&ErrUpdateFailed{ErrMarkedForDeletion})
	}
	// update with primary key
	const sqlstr = `UPDATE mapping SET ` +
		`source_id = $1 ` +
		`WHERE target_id = $2`
	// run
	logf(sqlstr, m.SourceID, m.TargetID)
	if _, err := db.ExecContext(ctx, sqlstr, m.SourceID, m.TargetID); err != nil {
		return logerror(err)
	}
	return nil
}

// Save saves the Mapping to the database.
func (m *Mapping) Save(ctx context.Context, db DB) error {
	if m.Exists() {
		return m.Update(ctx, db)
	}
	return m.Insert(ctx, db)
}

// Upsert performs an upsert for Mapping.
func (m *Mapping) Upsert(ctx context.Context, db DB) error {
	switch {
	case m._deleted: // deleted
		return logerror(&ErrUpsertFailed{ErrMarkedForDeletion})
	}
	// upsert
	const sqlstr = `INSERT INTO mapping (` +
		`target_id, source_id` +
		`) VALUES (` +
		`$1, $2` +
		`)` +
		` ON CONFLICT (target_id) DO ` +
		`UPDATE SET ` +
		`source_id = EXCLUDED.source_id `
	// run
	logf(sqlstr, m.TargetID, m.SourceID)
	if _, err := db.ExecContext(ctx, sqlstr, m.TargetID, m.SourceID); err != nil {
		return logerror(err)
	}
	// set exists
	m._exists = true
	return nil
}

// Delete deletes the Mapping from the database.
func (m *Mapping) Delete(ctx context.Context, db DB) error {
	switch {
	case !m._exists: // doesn't exist
		return nil
	case m._deleted: // deleted
		return nil
	}
	// delete with single primary key
	const sqlstr = `DELETE FROM mapping ` +
		`WHERE target_id = $1`
	// run
	logf(sqlstr, m.TargetID)
	if _, err := db.ExecContext(ctx, sqlstr, m.TargetID); err != nil {
		return logerror(err)
	}
	// set deleted
	m._deleted = true
	return nil
}

// MappingByTargetID retrieves a row from 'mapping' as a Mapping.
//
// Generated from index 'sqlite_autoindex_mapping_1'.
func MappingByTargetID(ctx context.Context, db DB, targetID string) (*Mapping, error) {
	// query
	const sqlstr = `SELECT ` +
		`target_id, source_id ` +
		`FROM mapping ` +
		`WHERE target_id = $1`
	// run
	logf(sqlstr, targetID)
	m := Mapping{
		_exists: true,
	}
	if err := db.QueryRowContext(ctx, sqlstr, targetID).Scan(&m.TargetID, &m.SourceID); err != nil {
		return nil, logerror(err)
	}
	return &m, nil
}

// MappingBySourceID retrieves a row from 'mapping' as a Mapping.
//
// Generated from index 'sqlite_autoindex_mapping_2'.
func MappingBySourceID(ctx context.Context, db DB, sourceID string) (*Mapping, error) {
	// query
	const sqlstr = `SELECT ` +
		`target_id, source_id ` +
		`FROM mapping ` +
		`WHERE source_id = $1`
	// run
	logf(sqlstr, sourceID)
	m := Mapping{
		_exists: true,
	}
	if err := db.QueryRowContext(ctx, sqlstr, sourceID).Scan(&m.TargetID, &m.SourceID); err != nil {
		return nil, logerror(err)
	}
	return &m, nil
}
